<?php

declare(strict_types = 1);

namespace Drupal\gutenberg_group_media\Persistence;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\file\Entity\File;
use Drupal\group\Plugin\Group\Relation\GroupRelationTypeManagerInterface;
use Drupal\group_finder\GroupFinderProviderInterface;
use Drupal\gutenberg\Persistence\MediaTypePersistenceManager;
use Drupal\gutenberg\Persistence\MediaTypePersistenceManagerInterface;
use Drupal\media\Entity\Media;

/**
 * Decorator media type persistence manager.
 */
class MediaTypePersistenceManagerDecorator extends MediaTypePersistenceManager {

  /**
   * The original media type persistence manager.
   *
   * @var \Drupal\gutenberg\Persistence\MediaTypePersistenceManagerInterface
   */
  protected MediaTypePersistenceManagerInterface $MediaTypePersistenceManager;

  /**
   * The group finder provider service.
   *
   * @var \Drupal\group_finder\GroupFinderProviderInterface
   */
  protected GroupFinderProviderInterface $groupFinderProvider;
  
  /**
	 * The group relation type manager.
	 *
	 * @var \Drupal\group\Plugin\Group\Relation\GroupRelationTypeManagerInterface
	 */
	 protected GroupRelationTypeManagerInterface $groupRelationTypeManager;

  /**
   * MediaTypePersistenceManagerDecorator constructor.
   *
   * @param \Drupal\gutenberg\Persistence\MediaTypePersistenceManagerInterface $media_type_persistence_manager
   *   The original media type persistence manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The current user.
   * @param \Drupal\group_finder\GroupFinderProviderInterface $group_finder_provider
   *   The group finder provider service.
	 * @param \Drupal\group\Plugin\Group\Relation\GroupRelationTypeManagerInterface $group_relation_type_manager
   *   The group relation type manager.
   */
  public function __construct(MediaTypePersistenceManagerInterface $media_type_persistence_manager, EntityTypeManagerInterface $entity_type_manager, AccountInterface $account, GroupFinderProviderInterface $group_finder_provider, GroupRelationTypeManagerInterface $group_relation_type_manager) {
    parent::__construct($entity_type_manager, $account);
    $this->MediaTypePersistenceManager = $media_type_persistence_manager;
    $this->groupFinderProvider = $group_finder_provider;
    $this->groupRelationTypeManager = $group_relation_type_manager;
  }
  
  /**
   * {@inheritDoc}
   */
  public function save(string $media_type, File $file_entity): ?Media {
    if (($group = $this->groupFinderProvider->get()?->getGroup()) && $group->id()) {
      $plugin_id = 'group_media:' . $media_type;
      $access = $this->groupRelationTypeManager->getAccessControlHandler($plugin_id)
        ->entityCreateAccess($group, \Drupal::currentUser());
      if ($access) {
        $media = $this->MediaTypePersistenceManager->save($media_type, $file_entity);
        $group->addRelationship($media, $plugin_id);
        return $media;
      }
      else {
        return NULL;
      }
    }

    return $this->MediaTypePersistenceManager->save($media_type, $file_entity);
  }

}
