<?php

declare(strict_types = 1);

namespace Drupal\gutenberg_group_media\Plugin\GroupFinder;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\group\Entity\GroupRelationship;
use Drupal\user\UserInterface;

/**
 * Plugin for gutenberg media routes originated from group by content.
 *
 * @GroupFinder(
 *   id = "gutenberg_media_routes_in_group_by_content",
 *   label = @Translation("Gutenberg media routes in group by content"),
 *   description = @Translation("When in context of gutenberg media routes originated from group by content."),
 *   weight = 70,
 * )
 */
class GutenbergMediaRoutesInGroupByContent extends GutenbergMediaRoutesBase {

  /**
   * {@inheritdoc}
   */
  public function isApplicable(): bool {
    $is_applicable = parent::isApplicable();

    if ($is_applicable && ($url_object = $this->getUrlObject())) {
      $route_name_parts = explode('.', $url_object->getRouteName());
      if (current($route_name_parts) !== 'entity') {
        return FALSE;
      }

      $entity_type = $route_name_parts[1];
      if (($parameters = $url_object->getRouteParameters()) && array_key_exists($entity_type, $parameters)) {
        $storage = $this->entityTypeManager->getStorage($entity_type);
        if ($entity = $storage?->load($parameters[$entity_type])) {
          // Exclude user entity type so that it's never handled by this
          // finder. In case you want to get specific a group by current user,
          // you should create your own custom group finder plugin or override
          // this plugin.
          if ($entity instanceof ContentEntityInterface
            && !$entity instanceof UserInterface
            && $entity->hasLinkTemplate('canonical')) {
            // Load all the group relationships for this entity.
            if ($group_relationships = GroupRelationship::loadByEntity($entity)) {
              if (count($group_relationships) >= 1) {
                // Use the first group assigned to the entity.
                // In case your website has a different behaviour, e.g. get
                // last group, then you should create your own custom group
                // finder plugin or override this plugin.
                $group_relationships = reset($group_relationships);
                $this->group = $group_relationships->getGroup();
                return TRUE;
              }
            }
          }
        }
      }
    }

    return FALSE;
  }

}
